function pos = convertGPXtoXY(gpx,centerAtZero)
    % pos = convertGPXtoXY(gpx,centerAtZero)
    %
    % Converts the previously loaded GPS angular coordinates from angular
    % to cartesian coordinates
    % 
    % inputs:
    %   gpx          =	matrix containing the angular gps coordinates and 
    %                   time
    %   centerAtZero =  true/false decides whether to centre the converted
    %                   xy coordinates around zero	
    % outputs:
    %   pos         =   structure containing GPS position (angular and
    %                   cartesian) and time information (date and timestamp)
    %
    % Author: Ricardo M.R. Ado,
    % Ultrafast Bio- and Nanophotonics group,
    % INL - International Iberian Nanotechnology Laboratory, Portugal
    % 
    % Software eveloped in frame of the Interreg NanoEATERS - Cdu#1 NanoLED
    % project, with funding grant number 0181_NANOEATERS_1_EP
    %
    % 
    
    % gpx file format:
    % col:  1   2   3   4   5   6   7   8   9  
    %       lat lon ele	yar mth	day	hor	min	s
    pos.gpx = gpx; % store raw data
    % retreive fields from the file
    pos.lat = gpx(:,1)*pi/180; pos.lon = gpx(:,2)*pi/180; pos.ele = gpx(:,3);
    pos.time.year = gpx(:,4); pos.time.month = gpx(:,5); pos.time.day = gpx(:,6);
    pos.time.hour = gpx(:,7); pos.time.min = gpx(:,8); pos.time.sec = gpx(:,9);
    pos.time.ms = zeros(size(gpx(:,9)));
    
    % calculate gps time since 1980-01-06
    pos.timeStamp = calculate_timeStamp(pos.time.year, pos.time.month,...
    pos.time.day, pos.time.hour, pos.time.min, pos.time.sec, pos.time.ms);
    
    pos.R0 = 6.3781e6; % (m) earth's radius (sea-level to core)
    pos.R = pos.R0 + pos.ele; % radius from earth's center over time
    
    % spherical coordinates
%     pos.y = pos.R .* cos(pos.lat) .* cos(pos.lon) * 1.3472;
%     pos.x = pos.R .* cos(pos.lat) .* sin(pos.lon) * 1.0486;
    % mercator projectin
    lon0 = 0.0098; %  greenwich reference
    pos.x = pos.R .* (pos.lon - lon0)*pi/180 * 41.6442;
    pos.y = pos.R .* log(tan(pi/4) + pos.lat/2) * 2.808;
    
    if centerAtZero
        pos.x = pos.x - pos.x(1); pos.y = pos.y - pos.y(1);
    end


    function timeStamp = calculate_timeStamp(year, month, day, hour, min, sec, ms)
        % calculate the time stamp: number of miliseconds since 1980:01:06
        % refernce day information
        ref = [];
        ref.timeStamp = 1451606400000; %2016-01-91  00:00:000
        ref.year = 2016; ref.month = 1; ref.day = 1;
        ref.hour = 0; ref.min = 0; ref.sec = 0; ref.ms = 0;
        daysPerMonth = [0 31 28 31 30 31 30 31 31 30 31 30 31];
        leapYear = zeros(100,1); leapYear(1:4:end) = 1;

        % calculate elapsed time since reference 
        dYear = year - ref.year;
        dHour = hour - ref.hour;
        dMin = min - ref.min;
        dSec = sec - ref.sec;
        dMs = ms - ref.ms;

        for ii = 1 : length(year)
            % number of days in the current year
            nDaysInYear = sum(daysPerMonth(1:month(ii))) +...
                sum(leapYear(1:(dYear(ii) + 1 - double(month(ii)<=2)))) + day(ii) - 1;

            % calculate timestamp
            timeStamp(ii,1) = dMs(ii) + dSec(ii)*1000 + dMin(ii)*60000 + dHour(ii) *3600000 +...
            nDaysInYear*86400000 + dYear(ii)*31536000000 + ref.timeStamp;
        end
    end

end