function [sensor, pos] = correlateSensorPos(sensor,pos,dt,pingMax)
    % [sensor, pos] = correlateSensorPos(sensor,pos,dt,pingMax)
    % 
    % Correlates the sensor measurements and gps position based on both
    % measurements time. The measurement temporal uniformization is done
    % via cubic interpolation.
    %
    % inputs:
    %   sensor  =   structure containing the sensor measurment values and
    %               timestamps
    %   pos     =   structure containing the gps position and time
    %               information
    %   dt      =   Uniform time step
    %   pingMax =   maximum supported ping value. This can be set to
    %               infivinity if no data limiting is intended
    % outpurs:
    %   sensor  =   updated sensor input
    %   pos     =   updates pos input
    %
    % Author: Ricardo M.R. Ado,
    % Ultrafast Bio- and Nanophotonics group,
    % INL - International Iberian Nanotechnology Laboratory, Portugal
    % 
    % Software eveloped in frame of the Interreg NanoEATERS - Cdu#1 NanoLED
    % project, with funding grant number 0181_NANOEATERS_1_EP
    %
    % 
    % generate uniform time array
    tStart = max([sensor.timeStamp(1,1) pos.timeStamp(1)]);
    tEnd = min([sensor.timeStamp(end,1) pos.timeStamp(end)]);
    time = tStart:dt:tEnd;
    time = time(:);
    
    % calculate sensor ping
    sensor.ping = diff(sensor.timeStamp); sensor.ping = [sensor.ping(1); sensor.ping];
    
    % interpolate values using new time array
    fields = fieldnames(sensor);
    for i = 1 : length(fields)
%     sensor.tempInt = interp1(sensor.timeStamp,sensor.temp,time,'cubic');
%     sensor.pingInt = interp1(sensor.timeStamp,sensor.ping,time,'cubic');
        sensor.([fields{i} 'Int']) = interp1(sensor.timeStamp,sensor.(fields{i}),time,'cubic');        
    end
    pos.xInt = interp1(pos.timeStamp,pos.x,time,'cubic');
    pos.yInt = interp1(pos.timeStamp,pos.y,time,'cubic');
    
    % store new time array in variables
    sensor.timeStampInt = time;
    pos.timeStampInt = time;
    
    % calculate distance from initial point
    pos.dist0Int = sqrt((pos.xInt - pos.xInt(1)).^2 + (pos.yInt - pos.yInt(1)).^2);
    
    sensor.pingInt(sensor.pingInt > pingMax) = pingMax;
end