function [X,Y,Z,ZAvg] = mapSpreadLine(pos,measure,xPx,yPx,integrateNtimes,center,maxDist)
    % [X,Y,Z,ZAvg] = mapSpreadLine(pos,measure,xPx,yPx,integrateNtimes,center,maxDist)
    %
    % Flood-fill-like interpolation algorithm to convert a recorded GPS
    % line strajectory into a 2D map
    %
    % imputs:
    %   pos             =   structure containing gps position and time
    %                       information
    %   measure         =   structure containing the measurement values and
    %                       timestamps
    %   xPx             =   array of x coordinates
    %   yPx             =   array of y coordinates
    %   integrateNtimes =   number of integration interpolations
    %   center          =   center position. Use [] to calculate the center
    %                       automaticlly
    %   maxDist         =   maximum distance. Use [] to calculate
    %                       automatically
    % outputs:
    %   X       =   meshgrid of x
    %   Y       =	meshgrid of y
    %   Z       =	map of line trajectory
    %   ZAvg    =	interpolated (flodd-filled) map
    % 
    % Author: Ricardo M.R. Ado,
    % Ultrafast Bio- and Nanophotonics group,
    % INL - International Iberian Nanotechnology Laboratory, Portugal
    % 
    % Software eveloped in frame of the Interreg NanoEATERS - Cdu#1 NanoLED
    % project, with funding grant number 0181_NANOEATERS_1_EP
    %
     
    % define where to find the x and y arrays
    x = pos.xInt;
    y = pos.yInt;
    
    % center on intial position
    x = x - x(1); y = y - y(1);
    
    % initialize image
    [X,Y] = meshgrid(xPx,yPx);
    Z = zeros(size(X));
    
    
    % find the approximate locastions within the Z image
    for ii = 1 : length(x)
        [~,xi] = min(abs(x(ii) - X(1,:)));
        [~,yi] = min(abs(y(ii) - Y(:,1)));
        Z(yi,xi) = measure(ii);
    end
    
    if ~isempty(center)
        dist = sqrt((X - center(1)).^2 + (Y - center(2)).^2);
        if ~isempty(maxDist)
            Z(dist > maxDist) = max(Z(:));
        else
            Z(dist > max([max(xPx) max(yPx)])) = max(Z(:));
        end
    end
    
    % average the nan values
    % neighbour pixels
    neigh = @(z,xx,yy) [z(yy-1,xx-1), z(yy,xx-1), z(yy+1,xx-1),...
                z(yy-1,xx), z(yy,xx), z(yy+1,xx),...
                z(yy-1,xx+1), z(yy,xx+1), z(yy+1,xx+1)];
    ZAvg = Z;
    ZAux = Z;
    % run the average process N times
    h = waitbar(0,'Integrating');
    for ii = 1 : integrateNtimes
        
        ZAux = cat(3,...
	ZAvg(2 -1: end-1 -1,2 -1:end-1 -1),    ZAvg(2 +0: end-1 +0,2 -1:end-1 -1),  ZAvg(2 +1: end-1 +1,2 -1:end-1 -1),...
    ZAvg(2 -1: end-1 -1,2 +0:end-1 +0),    ZAvg(2 +0: end-1 +0,2 +0:end-1 +0),  ZAvg(2 +1: end-1 +1,2 +0:end-1 +0),...
    ZAvg(2 -1: end-1 -1,2 +1:end-1 +1),    ZAvg(2 +0: end-1 +0,2 +1:end-1 +1),  ZAvg(2 +1: end-1 +1,2 +1:end-1 +1));
        ZAux = [zeros(1,size(Z,2) - 2,9); ZAux ; zeros(1,size(Z,2) - 2,9)];
        ZAux = [zeros(size(Z,1),1,9)      , ZAux , zeros(size(Z,1),1,9)];
        ZAvg = sum(ZAux,3)./sum(ZAux>0,3);
        ZAvg(isnan(ZAvg)) = 0;
        waitbar(ii/integrateNtimes,h);
        ZAvg(Z > 0) = Z(Z > 0);
        % ZAux = ZAvg;
    end
    close(h)
end